/* 
VTCInspector_v131.qml accompanying VTC inspector dll/dylib/so for BrainVoyager 23.2.
Latest update: 
24-06-28
By JM, update to QML JE/HB
*/

import QtQml
import QtQuick
import QtQuick.Layouts
import QtQuick.Controls

Pane {
    	id: root	
 	property real scale: 1.0
	width: 530 * root.scale
	height: 620 * root.scale

    ColumnLayout {
        id: mainLayout
	anchors {
		fill: parent
		margins: 2
		left: parent.left
		right: parent.right
	}	

        height: parent.height
        spacing: 2 // changing the spacing does not make a difference

        // from VTC inspector
        GroupBox {
                   id: grpbxOptions
                   title: "Options"
                   Layout.fillWidth: true
                   spacing: 0

		RowLayout {
                       id: colLayout

                   CheckBox {
                       id: chbDeleteOtherVMPs
                       text: qsTr("Delete Current VMPs")
                       width: 546
                       height: 20
                       x: 13
                       y: 30
                       checked: false
                   }

                   CheckBox {
                       id: chbAddVTCName
                       text: qsTr("Add the VTC name to the VMPs that will be created")
                       width: 546
                       height: 20
			x: 50
			y: 30
                       checked: false
                   }
                }
           }

        GroupBox {
                    id: grpbxMaps
                    title: "Create the following maps"
                    Layout.fillWidth: true
                    x: 12
                    y: 83
                    width: 565
                    height: 104

		anchors {
			top: grpbxOptions.bottom
		}

                //    Layout.preferredHeight: 104

                    ColumnLayout {
                        id: colLayout2
                        anchors.fill: parent

                    CheckBox {
                        id: chbCreateTSNRMap
                        text: qsTr("Temporal Signal-To-Noise")
                        width: 546
                        height: 20
                        x: 13 // position w.r.t. groupbox
                        y: 30
                        checked: false
                    }
                    Item { Layout.fillWidth: true } // horizontal spacer item

                    CheckBox {
                        id: chbCreateLaggedARMap
                        text: qsTr("1+2 Time Lagged Autocorrelation")
                        width: 546
                        height: 20
                        x: 13
                        y: 51
                        checked: false
                    }
                    Item { Layout.fillWidth: true } // horizontal spacer item

                    CheckBox {
                        id: chbCreateOutlierMap
                        text: qsTr("Outliers (maximum standard deviation in each voxel)")
                        width: 546
                        height: 20
                        x: 13
                        y: 72
                        checked: false
                    }
                    Item { Layout.fillWidth: true } // horizontal spacer item
                    }
            }

        GroupBox {
                id: grpbxParameters
                title: "Parameters"
                Layout.fillWidth: true

		anchors {
			top: grpbxMaps.bottom
		}

                ColumnLayout {
                    id: colLayout3
                    anchors.fill: parent

                    Label {
                        text: "Outlier Standard Deviation:"
                    }
                    SpinBox { // https://doc.qt.io/qt-6/qml-qtquick-controls2-spinbox.html
                        id: spbMaxDeviation

                      	editable: true
			from: 0
    			value: 400
    			to: 100 * 100
    			stepSize: 10//100

                        property int decimals: 2
                        property real realValue: value / 100

                        validator: DoubleValidator {
                            bottom: Math.min(spbMaxDeviation.from, spbMaxDeviation.to)
                            top:  Math.max(spbMaxDeviation.from, spbMaxDeviation.to)
                        }

                        textFromValue: function(value, locale) {
                            return Number(value / 100).toLocaleString(locale, 'f', spbMaxDeviation.decimals)
                        }

                        valueFromText: function(text, locale) {
                            return Number.fromLocaleString(locale, text) * 100
                        }
                    }
                    Item { Layout.fillWidth: true } // horizontal spacer item


                Label {
                    font.pixelSize: 8
                    text: "This parameter indicates the amount of deviation (in standard deviations) above which a time course value is considered as an outlier."
                }

                    Label {
                        text: "Spike Definition:"
                    }

                    SpinBox { 
                        id: spbSpikeDef

                      	editable: true
			from: 0
    			value: 300
    			to: 100 * 100
    			stepSize: 10//100

                        property int decimals: 2
                        property real realValue: value / 100

                        validator: DoubleValidator {
                            bottom: Math.min(spbSpikeDef.from, spbSpikeDef.to)
                            top:  Math.max(spbSpikeDef.from, spbSpikeDef.to)
                        }

                        textFromValue: function(value, locale) {
                            return Number(value / 100).toLocaleString(locale, 'f', spbSpikeDef.decimals)
                        }

                        valueFromText: function(text, locale) {
                            return Number.fromLocaleString(locale, text) * 100
                        }
                    }
                    Item { Layout.fillWidth: true } // horizontal spacer item

                Label {
                    font.pixelSize: 8
                    text: "Spike: if difference in standard deviations between the outlier and the average of it's neighbours is above the specified value"
                }
            }
        }

       GroupBox {
                id: grpbxTemporal
                title: "Temporal Information (outliers and spikes)"
                Layout.fillWidth: true

		anchors {
			top: grpbxParameters.bottom
		}

  		RowLayout {
                    id: colLayout4
                    anchors.fill: parent

             	   CheckBox {
             	       id: chbCreateTemporalOutlierLogTab
             	       text: qsTr("Write to Log Pane")
             	       checked: false
				x: 13
                	}

	                CheckBox {
	                    id: chbCreateTemporalOutlierFile
	                    text: qsTr("Write to CSV file")
	                    checked: false
				x: 50
                	}
                }
        }

       GroupBox {
                id: grpbxVTCFiles
                title: "List of VTC files (currently loaded VTC is used when list is empty):"
                Layout.fillWidth: true

		anchors {
			top: grpbxTemporal.bottom
		}
  
		ScrollView { // if this is not in the RowLayout, it will appear on top of the page
                id: view
                anchors.fill: parent
    		ScrollBar.horizontal.policy: ScrollBar.AlwaysOn
    		ScrollBar.vertical.policy: ScrollBar.AlwaysOn
		contentHeight: 70

                TextArea {
                    id: tbrFileList
                //    text: "VTC filenames"
                }
            }
	}

       RowLayout {

            id: buttonsRow

            Button {
                id: addVtcBtn
                text: "Add VTC"
                onClicked: root.onAdd()
            }
            Button {
                id: addMdmBtn
                text: "Add MDM"
                onClicked: root.onMDM()
            }
            Button {
                id: clearBtn
                text: "Clear List"
                onClicked: root.onClear()
            }

            Item { Layout.fillWidth: true } // horizontal spacer item

            Button {
                id: startBtn
                text: "Start"
                onClicked: root.onStart()
            }

            Button {
                id: quitBtn
                text: "Cancel"
                onClicked: BrainVoyager.FinishPlugin() // 'ClosePlugin' for pure qml scripts, but if qml dialog serves as frontend for plugin, 'FinishPlugin()' needs to be called!
            }
        }
	Layout.preferredHeight: contentItem.height
	Layout.minimumHeight: contentItem.height
    } // end Layout


    Component.onCompleted: {
  //      console.log("VTC Inspector plugin finished") 
  //      BrainVoyager.PrintToLog("VTC Inspector plugin finished.")
    }

    function onAdd() {

        BrainVoyager.PrintToLog("Script>  Dialog's Add VTC button clicked");
        BrainVoyager.SetPluginStringParameter("Command", "AddFile");
        BrainVoyager.ExecutePlugin();

        var newFileList = BrainVoyager.GetPluginStringParameter("FileList"); // get result from plugin

        BrainVoyager.PrintToLog(newFileList);
	tbrFileList.clear();
	tbrFileList.append(newFileList);

    }

    function onMDM() {

        BrainVoyager.PrintToLog("Script>  Dialog's Add MDM button clicked");
        BrainVoyager.SetPluginStringParameter("Command", "AddMDM");
        BrainVoyager.ExecutePlugin();
        var newFileList = BrainVoyager.GetPluginStringParameter("FileList"); // get result from plugin
        BrainVoyager.PrintToLog(newFileList);
	tbrFileList.clear(); 
	tbrFileList.append(newFileList);

    }

    function onClear() {
        BrainVoyager.PrintToLog("Script>  Dialog's Clear List button clicked");
	tbrFileList.clear();
    }

    function onStart()
    {
        BrainVoyager.PrintToLog("Script>  Dialog info to Brainvoyager");
        BrainVoyager.SetPluginStringParameter("Command", "Start");

        BrainVoyager.SetPluginIntParameter("Delete_Other_Vmps", chbDeleteOtherVMPs.checked);
        BrainVoyager.SetPluginIntParameter("Add_Vtc_Name", chbAddVTCName.checked);

        BrainVoyager.SetPluginIntParameter("Create_TSNR_Map", chbCreateTSNRMap.checked);
        BrainVoyager.SetPluginIntParameter("Create_Lagged_Autocor_Map", chbCreateLaggedARMap.checked);
        BrainVoyager.SetPluginIntParameter("Create_Outlier_Map", chbCreateOutlierMap.checked);

        BrainVoyager.SetPluginFloatParameter("Max_Deviation", spbMaxDeviation.realValue); 
        BrainVoyager.SetPluginFloatParameter("Spike_Def", spbSpikeDef.realValue); 
        BrainVoyager.SetPluginIntParameter("Create_Temporal_Outlier_Logtab", chbCreateTemporalOutlierLogTab.checked);
        BrainVoyager.SetPluginIntParameter("Create_Temporal_Outlier_File", chbCreateTemporalOutlierFile.checked);
        BrainVoyager.ExecutePlugin();
	BrainVoyager.FinishPlugin();
    }
}
